pub mod docker_auth;
pub mod docker_config;

use serde::{Deserialize, Serialize};
use crate::{DockerTagsFetcher, QueryArgs};
use crate::registry::docker_config::DockerPassword;

/// registry api response struct
#[derive(Serialize, Deserialize)]
struct RegistryTagsResponse {
    name: Option<String>,
    tags: Option<Vec<String>>,
}

pub struct RegistryTagsFetcher {
    username: String,
    password: String,
}

impl RegistryTagsFetcher {
    pub fn new(docker_password: &DockerPassword) -> RegistryTagsFetcher {
        RegistryTagsFetcher {
            username: docker_password.username.to_string(),
            password: docker_password.password.to_string(),
        }
    }
}

impl DockerTagsFetcher for RegistryTagsFetcher {
    fn get_tags(&self, args: &QueryArgs) -> Vec<String> {
        let namespace = args.namespace.as_str();
        let repository = args.repository.as_str();
        let token = docker_auth::get_token(namespace, repository, self.username.as_str(), self.password.as_str()).unwrap();
        let url = format!("https://registry-1.docker.io/v2/{namespace}/{repository}/tags/list");
        let client = reqwest::blocking::Client::new();
        let response = client.get(url)
            .header("Authorization", format!("Bearer {token}"))
            .send();
        let mut results = Vec::<String>::new();
        if let Ok(r) = response {
            let json: RegistryTagsResponse = serde_json::from_str(r.text().unwrap().as_str()).unwrap();
            match json.tags {
                Some(mut x) => { results.append(&mut x); }
                None => { results.push(format!("Image not found: {}/{}", namespace, repository)); }
            }
        };
        results
    }
}
