use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize)]
struct RegistryTokenResponse {
    token: Option<String>,
    access_token: Option<String>,
    expires_in: u32,
    issued_at: Option<String>,
}

/// get the access token for tags request
pub fn get_token(namespace: &str, repository: &str, username: &str, password: &str) -> Option<String> {
    let service = "registry.docker.io";
    let scope = format!("repository:{namespace}/{repository}:pull");
    let auth = format!("https://auth.docker.io/token?service={service}&scope={scope}");
    let client = reqwest::blocking::Client::new();
    let response = client.get(auth)
        .basic_auth(username, Some(password))
        .send();
    match response {
        Ok(r) => {
            let json: RegistryTokenResponse = serde_json::from_str(r.text().unwrap().as_str()).unwrap();
            Some(json.token.unwrap())
        }
        Err(_) => { None }
    }
}
